#include <fstream>
#include <vector>
#include <queue>
#include <algorithm>
using namespace std;

ifstream fin("dijkstra.in");
ofstream fout("dijkstra.out");

struct Pair {
	int node, dist;
	bool operator < (const Pair& p) const
	{
		return dist > p.dist;
	}
};

const int Inf = 0x3f3f3f3f;

using PI  = pair<int, int>;
using VI  = vector<int>;
using VP  = vector<PI>;
using VVP = vector<VP>;

VI d;
VVP G; // graful
int n;

void ReadGraph();
void Dijkstra(int x, VI& d);
void WriteDistances();


int main()
{
	ReadGraph();
	Dijkstra(1, d);
	WriteDistances();
}

void Dijkstra(int x, VI& d)
{
	d = VI(n + 1, Inf);
	priority_queue<Pair> Q; // min HEAP
	d[x] = 0;
	Q.push({x, 0});
	int y, w, dist;
	while (!Q.empty())
	{
		x = Q.top().node;
		dist = Q.top().dist;
		Q.pop();
		
		if (dist > d[x])
			continue;
		
		for (auto& p : G[x])
		{
			y = p.first;
			w = p.second;
			if (d[y] > d[x] + w)
			{
				d[y] = d[x] + w;
				Q.push({y, d[y]});
			}
		}	
	}	
}

void ReadGraph()
{
	int x, y, w, m;
	fin >> n >> m;
	G = VVP(n + 1);
	
	while (m--)
	{
		fin >> x >> y >> w;
		G[x].emplace_back(y, w);
		G[y].emplace_back(x, w);
	}
}

void WriteDistances()
{
	for (int i = 1; i <= n; ++i)
		fout << d[i] << ' ';
}







